@@ -34,13 +34,16 @@ using SemWeb;
 
 namespace Beagle.Filters {
 
+	[PropertyKeywordMapping (Keyword="imagetag", PropertyName="image:tag", IsKeyword=false, Description="FSpot, Digikam image tags")]
+	[PropertyKeywordMapping (Keyword="comment", PropertyName="fixme:comment", IsKeyword=false, Description="User comments")]
 	public abstract class FilterImage : Beagle.Daemon.Filter {
 
 		public FilterImage ()
 		{
 			// 1: Base
 			// 2: Added fspot:IsIndexed field, added width & height properties
-			SetVersion (2);
+			// 3: Add Digikam tags and caption
+			SetVersion (3);
 		}
 
 		protected virtual void PullImageProperties () { }
@@ -51,14 +54,17 @@ namespace Beagle.Filters {
 
 		protected int Width {
 			set { width = value; }
+			get { return width; }
 		}
 
 		protected int Height {
 			set { height = value; }
+			get { return height; }
 		}
 
 		protected int Depth {
 			set { depth = value; }
+			get { return depth; }
 		}
 
 		protected override void DoPullProperties ()
@@ -66,63 +72,100 @@ namespace Beagle.Filters {
 			PullImageProperties ();
 
 			if (width > 0)
-				AddProperty (Beagle.Property.NewKeyword ("fixme:width", width));
+				AddProperty (Beagle.Property.NewUnsearched ("fixme:width", width));
 
 			if (height > 0)
-				AddProperty (Beagle.Property.NewKeyword ("fixme:height", height));
+				AddProperty (Beagle.Property.NewUnsearched ("fixme:height", height));
 
 			if (depth > 0)
-				AddProperty (Beagle.Property.NewKeyword ("fixme:depth", depth));
+				AddProperty (Beagle.Property.NewUnsearched ("fixme:depth", depth));
 
 			try {
-				FSpotTools.Photo photo = FSpotTools.GetPhoto (this.FileInfo.FullName);
+				AddFSpotInformation (this.FileInfo.FullName);
+				AddDigikamInformation (this.FileInfo.FullName);
+			} catch (Exception ex) {
+				Logger.Log.Error ("Exception trying to retrieve FSpot/Digikam information:" + ex);
+			}
 
-				if (photo == null)
-					return;
 
-				AddProperty (Beagle.Property.NewBool ("fspot:IsIndexed", true));
-				
-				if (photo.Description != null && photo.Description != "")
-					AddProperty (Beagle.Property.New ("fspot:Description", photo.Description));
+		}
+
+		private void AddFSpotInformation (string path)
+		{
+			FSpotTools.Photo photo = FSpotTools.GetPhoto (this.FileInfo.FullName);
+
+			if (photo == null)
+				return;
+
+			AddProperty (Beagle.Property.NewBool ("fspot:IsIndexed", true));
 			
-				foreach (FSpotTools.Tag tag in photo.Tags) {				
-					if (tag.Name != null && tag.Name != "")
-						AddProperty (Beagle.Property.New ("fspot:Tag", tag.Name));
+			if (photo.Description != null && photo.Description != "") {
+				AddProperty (Beagle.Property.New ("fspot:Description", photo.Description));
+				AddProperty (Beagle.Property.NewUnstored ("fixme:comment", photo.Description));
+			}
+			
+			foreach (FSpotTools.Tag tag in photo.Tags) {				
+				if (tag.Name != null && tag.Name != "") {
+					AddProperty (Beagle.Property.New ("fspot:Tag", tag.Name));
+					AddProperty (Beagle.Property.NewUnstored ("image:tag", tag.Name));
 				}
-			} catch {
-				//Console.WriteLine ("Failed extracting F-Spot information for '{0}'", this.FileInfo.Name);
+			}
+		}
+
+		private void AddDigikamInformation (string path)
+		{
+			DigikamTags.DigikamData digikam_data = DigikamTags.GetDigikamData (this.FileInfo.FullName);
+
+			if (digikam_data == null)
+				return;
+
+			AddProperty (Beagle.Property.NewBool ("digikam:IsIndexed", true));
+			
+			if (digikam_data.caption != null && digikam_data.caption != "") {
+				AddProperty (Beagle.Property.New ("digikam:caption", digikam_data.caption));
+				AddProperty (Beagle.Property.NewUnstored ("fixme:comment", digikam_data.caption));
+			}
+			
+			foreach (string tag in digikam_data.Tags) {
+				AddProperty (Beagle.Property.New ("digikam:Tag", tag));
+				AddProperty (Beagle.Property.NewUnstored ("image:tag", tag));
 			}
 		}
 		
-		protected void AddXmpProperties (XmpFile xmp)
+		internal void AddXmpProperties (XmpFile xmp)
 		{
 			Resource subject_anon = null;
 			Resource creator_anon = null;
-			
+			Resource rights_anon = null;
+
 			foreach (Statement stmt in xmp.Store) {
 				if (stmt.Predicate == MetadataStore.Namespaces.Resolve ("dc:subject")) {
-					System.Console.WriteLine ("found subject");
+					//Console.WriteLine ("found subject");
 					subject_anon = stmt.Object;
 				} else if (stmt.Predicate == MetadataStore.Namespaces.Resolve ("dc:creator")) {
-					System.Console.WriteLine ("found creator");
+					//Console.WriteLine ("found creator");
 					creator_anon = stmt.Object;
 				} else if (stmt.Predicate == MetadataStore.Namespaces.Resolve ("dc:rights")) {
-					AddProperty (Beagle.Property.New ("dc:rights", ((Literal)stmt.Object).Value));
+					rights_anon = stmt.Object;
 				} else if (stmt.Predicate == MetadataStore.Namespaces.Resolve ("dc:title")) {
 					AddProperty (Beagle.Property.New ("dc:title", ((Literal)stmt.Object).Value));
 				} else if (stmt.Predicate == MetadataStore.Namespaces.Resolve ("tiff:Model")) {
 					// NOTE: the namespaces for xmp and beagle don't always match up
 					AddProperty (Beagle.Property.New ("exif:Model", ((Literal)stmt.Object).Value));
-					}
+				}
 			}
 			
 			foreach (Statement stmt in xmp.Store) {
 				if (stmt.Subject == subject_anon && 
 				    stmt.Predicate != MetadataStore.Namespaces.Resolve ("rdf:type")) {
 					AddProperty (Beagle.Property.New ("dc:subject", ((Literal)stmt.Object).Value));
-				} else if (stmt.Subject == creator_anon && 
-					   stmt.Predicate != MetadataStore.Namespaces.Resolve ("rdf:type"))
+				} else if (stmt.Subject == creator_anon &&  
+					   stmt.Predicate != MetadataStore.Namespaces.Resolve ("rdf:type")) {
 					AddProperty (Beagle.Property.New ("dc:creator", ((Literal)stmt.Object).Value));
+				} else if (stmt.Subject == rights_anon &&  
+					   stmt.Predicate != MetadataStore.Namespaces.Resolve ("rdf:type")) {
+					AddProperty (Beagle.Property.New ("dc:rights", ((Literal)stmt.Object).Value));
+				}
 			}
 		}
 	}
