@@ -33,6 +33,7 @@ using System.Net;
 using Beagle;
 using Beagle.Util;
 using Beagle.Daemon;
+using Hit = Beagle.Hit;
 
 using Lucene.Net.Index;
 using Lucene.Net.Search;
@@ -256,7 +257,7 @@ class DumpIndexTool {
 			return;
 		
 		// else query by path - this is for static indexes
-		path = StringFu.PathToQuotedFileUri (path);
+		path = UriFu.PathToFileUriString (path);
 		Console.WriteLine ("Querying by:[" + path + "]");
 		query = new TermQuery(new Term("Uri", path));
 		DoQuery (driver, query);
