@@ -190,7 +190,7 @@ namespace Beagle.Daemon.KMailQueryable {
 			string body = message.GetBody (true, out html);
 			// FIXME: Also handle snippets from html message parts - involves invoking html filter
 			if (html) {
-				Logger.Log.Debug ("No text/plain message part in " + hit.UriAsString);
+				Logger.Log.Debug ("No text/plain message part in " + hit.Uri);
 				message.Dispose ();
 				return null;
 			}
@@ -309,15 +309,20 @@ namespace Beagle.Daemon.KMailQueryable {
 				StreamReader reader = new StreamReader (kmailrc);
 				string section = "";
 				string line;
-				while ((line = reader.ReadLine ()) != null) {
-					if (line.StartsWith ("[") && line.EndsWith ("]")) {
-						section = line;
-					}
-					if (section == "[General]") {
-						if (line.StartsWith ("folders=") && line.Length > 8) {
-							return StringFu.ExpandEnvVariables (line.Substring(8));
+
+				try {
+					while ((line = reader.ReadLine ()) != null) {
+						if (line.StartsWith ("[") && line.EndsWith ("]")) {
+							section = line;
+						}
+						if (section == "[General]") {
+							if (line.StartsWith ("folders=") && line.Length > 8) {
+								return StringFu.ExpandEnvVariables (line.Substring(8));
+							}
 						}
 					}
+				} finally {
+					reader.Close ();
 				}
 			}
 
