@@ -158,8 +158,7 @@ namespace Beagle.Daemon {
 			try {
 				ShutdownHook ();
 			} catch (Exception ex) {
-				Logger.Log.Warn ("Caught exception in shutdown hook");
-				Logger.Log.Warn (ex);
+				Logger.Log.Warn (ex, "Caught exception in shutdown hook");
 			}
 		}
 
@@ -281,11 +280,8 @@ namespace Beagle.Daemon {
 
 			if (change_data != null) {
 				
-				if (change_data.RemovedUris != null) {
-					foreach (Uri uri in change_data.RemovedUris)
-						Log.Debug ("Signalling removal of {0}", uri);
+				if (change_data.RemovedUris != null)
 					query_result.Subtract (change_data.RemovedUris);
-				}
 
 				// If nothing was added, we can safely return now: this change
 				// cannot have any further effect on an outstanding live query.
@@ -300,12 +296,35 @@ namespace Beagle.Daemon {
 				if (query.IsIndexListener) {
 					ArrayList synthetic_hits = new ArrayList ();
 					foreach (Uri uri in added_uris) {
-						if (our_uri_filter != null && ! our_uri_filter (uri))
-							continue;
+						if (our_uri_filter != null) {
+							bool accept = false;
+
+							try {
+								accept = our_uri_filter (uri);
+							} catch (Exception e) {
+								Log.Warn (e, "Caught an exception in HitIsValid for {0}", uri);
+							}
+
+							if (! accept)
+								continue;
+						}
+
 						Hit hit = new Hit ();
 						hit.Uri = uri;
-						if (our_hit_filter != null && ! our_hit_filter (hit))
-							continue;
+
+						if (our_hit_filter != null) {
+							bool accept = false;
+
+							try {
+								accept = our_hit_filter (hit);
+							} catch (Exception e) {
+								Log.Warn (e, "Caught an exception in HitFilter for {0}", hit.Uri);
+							}
+
+							if (! accept)
+								continue;
+						}
+
 						synthetic_hits.Add (hit);
 					}
 					if (synthetic_hits.Count > 0)
@@ -430,7 +449,7 @@ namespace Beagle.Daemon {
 		{
 			string path = Path.Combine (Path.Combine (PathFinder.IndexDir, this.IndexName), name);
 			
-			return new FileStream (path, System.IO.FileMode.Create, FileAccess.Write);
+			return new FileStream (path, System.IO.FileMode.Create, FileAccess.Write, FileShare.ReadWrite);
 		}
 
 		public void WriteDataLine (string name, string line)
@@ -532,11 +551,6 @@ namespace Beagle.Daemon {
 			LuceneQueryable queryable;
 			IIndexableGenerator generator;
 
-			// Hook to be invoked after the IIndexableGenerator
-			// has finished processing a batch of Indexables,
-			// just prior to flushing the driver.
-			Scheduler.Hook pre_flush_hook;
-
 			public AddGeneratorTask (LuceneQueryable     queryable,
 						 IIndexableGenerator generator)
 			{
@@ -554,9 +568,14 @@ namespace Beagle.Daemon {
 				QueryableState old_state = queryable.State;
 				queryable.State = QueryableState.Indexing;
 
+				// Number of times a null indexable was returned.  We don't want
+				// to spin tightly in a loop here if we're not actually indexing
+				// things.
+				int misfires = 0;
+
 				do {
 					if (! generator.HasNextIndexable ()) {
-						// ...except if there is no more work to do, of course.
+						// Of course, don't reschedule if there is no more work to do.
 						Reschedule = false;
 						break;
 					}
@@ -568,8 +587,14 @@ namespace Beagle.Daemon {
 					// This means that the generator didn't have an indexable
 					// to return this time through, but it does not mean that
 					// its processing queue is empty.
-					if (generated == null)
-						continue;
+					if (generated == null) {
+						misfires++;
+
+						if (misfires > 179) // Another totally arbitrary number
+							break;
+						else
+							continue;
+					}
 
 					if (queryable.PreAddIndexableHook (generated))
 						queryable.AddIndexable (generated);
@@ -707,6 +732,13 @@ namespace Beagle.Daemon {
 		//////////////////////////////////////////////////////////////////////////////////
 
 		// Optimize the index
+
+		private DateTime last_optimize_time = DateTime.MinValue;
+
+		public DateTime LastOptimizeTime {
+			get { return last_optimize_time; }
+			set { last_optimize_time = value; }
+		}
 		
 		private class OptimizeTask : Scheduler.Task {
 			LuceneQueryable queryable;
@@ -719,6 +751,7 @@ namespace Beagle.Daemon {
 			override protected void DoTaskReal ()
 			{
 				queryable.Optimize ();
+				queryable.LastOptimizeTime = DateTime.Now;
 			}
 		}
 
@@ -744,7 +777,14 @@ namespace Beagle.Daemon {
 		private void ScheduleOptimize ()
 		{
 			double optimize_delay;
-			optimize_delay = 10.0; // minutes
+
+			// Really we only want to optimize at most once a day, even if we have
+			// indexed a ton of dat
+			TimeSpan span = DateTime.Now - last_optimize_time;
+			if (span.TotalDays > 1.0)
+				optimize_delay = 10.0; // minutes;
+			else
+				optimize_delay = (new TimeSpan (TimeSpan.TicksPerDay) - span).TotalMinutes;
 
 			if (our_optimize_task == null)
 				our_optimize_task = NewOptimizeTask ();
@@ -892,9 +932,8 @@ namespace Beagle.Daemon {
 						// Map from internal->external Uris in the PostAddHook
 						PostAddHook (flushed_request.GetByUri (r.Uri), r);
 					} catch (Exception ex) {
-						Logger.Log.Warn ("Caught exception in PostAddHook '{0}' '{1}' '{2}'",
+						Logger.Log.Warn (ex, "Caught exception in PostAddHook '{0}' '{1}' '{2}'",
 								 r.Uri, r.FilterName, r.FilterVersion);
-						Logger.Log.Warn (ex);
 					}
 
 					// Every added Uri also needs to be listed as removed,
@@ -917,9 +956,8 @@ namespace Beagle.Daemon {
 					try {
 						PostRemoveHook (flushed_request.GetByUri (r.Uri), r);
 					} catch (Exception ex) {
-						Logger.Log.Warn ("Caught exception in PostRemoveHook '{0}'",
+						Logger.Log.Warn (ex, "Caught exception in PostRemoveHook '{0}'",
 								 r.Uri);
-						Logger.Log.Warn (ex);
 					}
 
 					// Add the removed Uri to the list for our
@@ -940,8 +978,7 @@ namespace Beagle.Daemon {
 						} catch (InvalidOperationException ex) {
 							// Queryable does not support adding children
 						} catch (Exception ex) {
-							Logger.Log.Warn ("Caught exception in PreChildAddHook '{0}'", child.DisplayUri);
-							Logger.Log.Warn (ex);
+							Logger.Log.Warn (ex, "Caught exception in PreChildAddHook '{0}'", child.DisplayUri);
 						}
 
 						if (please_add_a_new_task) {
