@@ -25,6 +25,7 @@
 //
 
 using System;
+using System.Collections;
 using System.Xml.Serialization;
 
 using Beagle.Util;
@@ -33,8 +34,11 @@ namespace Beagle.Daemon {
 
 	public class RemoteIndexerResponse : ResponseMessage {
 
+		[XmlAttribute]
 		public int ItemCount = -1;
 
+		public IndexerReceipt [] Receipts;
+
 		public RemoteIndexerResponse ()
 		{
 
