@@ -13,6 +13,7 @@ namespace Search {
 
 	public class MainWindow : Window {
 
+		UIManager uim;
 		Search.GroupView view;
 		Search.Entry entry;
 		Gtk.Button button;
@@ -30,9 +31,10 @@ namespace Search {
 
 		string queryText;
 		Beagle.Query currentQuery;
-		Search.ScopeType scope = ScopeType.Everywhere;
+		Search.ScopeType scope = ScopeType.Everything;
 		Search.SortType sort = SortType.Modified;
 		Search.TypeFilter filter = null;
+		bool showDetails = true;
 
 		XKeybinder keybinder = new XKeybinder ();
 
@@ -128,7 +130,6 @@ namespace Search {
 			Title = "Desktop Search";
 			Icon = Beagle.Images.GetPixbuf ("system-search.png");
 
-			BorderWidth = 3;
 			DefaultWidth = 700;
 			DefaultHeight = 550;
 			DeleteEvent += OnWindowDelete;
@@ -136,9 +137,10 @@ namespace Search {
 			VBox vbox = new VBox ();
 			vbox.Spacing = 3;
 
-			UIManager uim = new UIManager (this);
+			uim = new UIManager (this);
 			uim.ScopeChanged += OnScopeChanged;
 			uim.SortChanged += OnSortChanged;
+			uim.ToggleDetails += OnToggleDetails;
 			uim.ShowQuickTips += OnShowQuickTips;
 			vbox.PackStart (uim.MenuBar, false, false, 0);
 
@@ -181,6 +183,7 @@ namespace Search {
 			pages = new Gtk.Notebook ();
 			pages.ShowTabs = false;
 			pages.ShowBorder = false;
+			pages.BorderWidth = 3;
 			vbox.PackStart (pages, true, true, 0);
 
 			quicktips = new Pages.QuickTips ();
@@ -202,6 +205,7 @@ namespace Search {
 
 			view = new GroupView ();
 			view.TileSelected += ShowInformation;
+			view.CategoryToggled += OnCategoryToggled;
 			panes.MainContents = view;
 			
 			Add (vbox);
@@ -343,17 +347,45 @@ namespace Search {
 			}
 		}
 
-		private void OnScopeChanged (Search.ScopeType newScope)
+		private void OnScopeChanged (Search.ScopeType toggled, bool active)
 		{
-			view.Scope = scope = newScope;
+			if (active)
+				view.Scope = scope = scope | toggled;
+			else
+				view.Scope = scope = scope ^ toggled;
+			
 			CheckNoMatch ();
 		}
+		
+		private void OnCategoryToggled (ScopeType toggled)
+		{
+			string name =  ScopeType.GetName (typeof (ScopeType), toggled);
+			try {
+				ToggleAction act = (ToggleAction) uim.GetAction ("/ui/MenuBar/Search/Scope/" +  name);
+				act.Active = ! act.Active;
+			}
+			catch (Exception ex) {
+				Console.WriteLine("Exception caught when trying to deactivate menu entry {0}:",name);
+				Console.WriteLine(ex);
+				return;
+			}
+		}
+		
 
 		private void OnSortChanged (Search.SortType newSort)
 		{
 			view.Sort = sort = newSort;
 		}
 
+		private void OnToggleDetails (bool active)
+		{
+			showDetails = active;
+			if (panes.Details != null)
+				panes.ToggleDetails (showDetails);
+			else
+				panes.ToggleDetails (false);
+		}
+
 		private void OnShowQuickTips ()
 		{
 			if (currentQuery != null) {
@@ -368,10 +400,16 @@ namespace Search {
 
 		private void ShowInformation (Tiles.Tile tile)
 		{
-			if (tile != null)
+			if (tile != null) {
 				panes.Details = tile.Details;
-			else
+				if (tile.Details != null)
+					panes.ToggleDetails (showDetails);
+				else
+					panes.ToggleDetails (false);
+			} else {
 				panes.Details = null;
+				panes.ToggleDetails (false);
+			}
 		}
 
 		private void OnFinished (FinishedResponse response)
